SystemOrganization addCategory: #TusLibros!

Collection subclass: #Cart
	instanceVariableNames: 'contents catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Cart class
	instanceVariableNames: 'catolog catalog'!

Cart class
	instanceVariableNames: 'catolog catalog'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/10/2012 20:25'!
catalog
	^catalog.! !

!Cart class methodsFor: 'instance creation' stamp: 'Maffongelli 6/10/2012 17:37'!
contentsClass
	^Dictionary! !

!Cart class methodsFor: 'instance creation' stamp: 'Maffongelli 6/10/2012 19:41'!
new
	catalog:= Catalog new.
	^ self new: 4! !

!Cart class methodsFor: 'instance creation' stamp: 'Maffongelli 6/10/2012 17:37'!
new: nElements
	^ super new setContents: (self contentsClass new: nElements)! !

!Cart class methodsFor: 'instance creation' stamp: 'Maffongelli 6/10/2012 17:37'!
newFrom: aCollection 
	"Answer an instance of me containing the same elements as aCollection."

	^ self withAll: aCollection

"Examples:
	Bag newFrom: {1. 2. 3. 3}
	{1. 2. 3. 3} as: Bag
"! !

!Cart class methodsFor: 'instance creation' stamp: 'Maffongelli 6/10/2012 19:37'!
newWithCatalog: aCatolog
	catolog := aCatolog.
	^ self new: 4! !

!Cart methodsFor: 'comparing' stamp: 'Maffongelli 6/10/2012 17:37'!
= aBag
	"Two bags are equal if
	 (a) they are the same 'kind' of thing.
	 (b) they have the same size.
	 (c) each element occurs the same number of times in both of them"

	(aBag isKindOf: Bag) ifFalse: [^false].
	self size = aBag size ifFalse: [^false].
	contents associationsDo: [:assoc|
		(aBag occurrencesOf: assoc key) = assoc value
			ifFalse: [^false]].
	^true

! !

!Cart methodsFor: 'adding' stamp: 'Maffongelli 6/10/2012 17:37'!
add: newObject 
	"Include newObject as one of the receiver's elements. Answer newObject."

	^ self add: newObject withOccurrences: 1! !

!Cart methodsFor: 'adding' stamp: 'Maffongelli 6/10/2012 17:37'!
add: newObject withOccurrences: anInteger 
	"Add newObject anInteger times to the receiver. Answer newObject."

	contents at: newObject put: (contents at: newObject ifAbsent: [0]) + anInteger.
	^ newObject! !

!Cart methodsFor: 'converting' stamp: 'Maffongelli 6/10/2012 17:37'!
asBag
	^ self! !

!Cart methodsFor: 'converting' stamp: 'Maffongelli 6/10/2012 17:37'!
asSet
	"Answer a set with the elements of the receiver."

	^ contents keys asSet! !

!Cart methodsFor: 'accessing' stamp: 'Maffongelli 6/10/2012 17:37'!
at: index 
	self errorNotKeyed! !

!Cart methodsFor: 'accessing' stamp: 'Maffongelli 6/10/2012 17:37'!
at: index put: anObject 
	self errorNotKeyed! !

!Cart methodsFor: 'accessing' stamp: 'Maffongelli 6/10/2012 17:37'!
cumulativeCounts
	"Answer with a collection of cumulative percents covered by elements so far."
	| s n |
	s := self size / 100.0. n := 0.
	^ self sortedCounts asArray collect:
		[:a | n := n + a key. (n / s roundTo: 0.1) -> a value]! !

!Cart methodsFor: 'enumerating' stamp: 'Maffongelli 6/10/2012 17:37'!
do: aBlock 
	"Refer to the comment in Collection|do:."

	contents associationsDo: [:assoc | assoc value timesRepeat: [aBlock value: assoc key]]! !

!Cart methodsFor: 'testing' stamp: 'Maffongelli 6/10/2012 17:37'!
includes: anObject 
	"Refer to the comment in Collection|includes:."

	^contents includesKey: anObject! !

!Cart methodsFor: 'testing' stamp: 'Maffongelli 6/10/2012 17:37'!
occurrencesOf: anObject 
	"Refer to the comment in Collection|occurrencesOf:."

	(self includes: anObject)
		ifTrue: [^contents at: anObject]
		ifFalse: [^0]! !

!Cart methodsFor: 'copying' stamp: 'Maffongelli 6/10/2012 17:37'!
postCopy
	super postCopy.
	contents := contents copy! !

!Cart methodsFor: 'removing' stamp: 'Maffongelli 6/10/2012 17:37'!
remove: oldObject ifAbsent: exceptionBlock 
	"Refer to the comment in Collection|remove:ifAbsent:."

	| count |
	count := contents at: oldObject ifAbsent: [^ exceptionBlock value].
	count = 1
		ifTrue: [contents removeKey: oldObject]
		ifFalse: [contents at: oldObject put: count - 1].
	^ oldObject! !

!Cart methodsFor: 'removing' stamp: 'Maffongelli 6/10/2012 17:37'!
removeAll
	"Implementation Note: as contents will be overwritten, a shallowCopy of self would be modified.
	An alternative implementation preserving capacity would be to create a new contents:
	self setContents: (self class contentsClass new: contents size)."
	
	contents removeAll! !

!Cart methodsFor: 'private' stamp: 'Maffongelli 6/10/2012 17:37'!
setContents: aDictionary
	contents := aDictionary! !

!Cart methodsFor: 'accessing' stamp: 'Maffongelli 6/10/2012 17:37'!
size
	"Answer how many elements the receiver contains."

	| tally |
	tally := 0.
	contents do: [:each | tally := tally + each].
	^ tally! !

!Cart methodsFor: 'accessing' stamp: 'Maffongelli 6/10/2012 17:37'!
sortedCounts
	"Answer with a collection of counts with elements, sorted by decreasing
	count."
	"Suggested by l. Uzonyi"

	^(Array new: contents size streamContents: [ :stream |
 			contents associationsDo: [ :each |
 				stream nextPut: each value -> each key ] ])
 		sort: [:x :y | x >= y ];
 		yourself! !

!Cart methodsFor: 'accessing' stamp: 'Maffongelli 6/10/2012 17:37'!
sortedElements
	"Answer with a collection of elements with counts, sorted by element."
	"Suggested by l. Uzonyi"
	
	^contents associations
 		sort;
 		yourself! !

!Cart methodsFor: 'accessing' stamp: 'Maffongelli 6/10/2012 17:37'!
valuesAndCounts

	^ contents! !

Object subclass: #Book
	instanceVariableNames: 'ISBN'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Book class methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/10/2012 16:22'!
isbn: aIsbn

 ^(self new) isbn: aIsbn.! !

!Book class methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/10/2012 17:55'!
new: aISBN

	isbn := aISBN.
	! !

!Book methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/10/2012 16:20'!
isbn

  ^ISBN! !

!Book methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/10/2012 16:23'!
isbn: aIsbn

  ISBN:= aIsbn.! !

Object subclass: #Catalog
	instanceVariableNames: 'books'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Catalog methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/10/2012 16:33'!
add: aBook

	books add: aBook.! !

!Catalog methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/10/2012 16:38'!
contains: aBook

 ^books includes: aBook.! !

!Catalog methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/10/2012 16:41'!
initialize

	books:= Set new.! !

!Catalog methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/10/2012 16:39'!
isEmpty 

 ^books isEmpty.! !

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/10/2012 17:52'!
newBook
	
	^Book new:01.! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/10/2012 17:43'!
newCart
	
	^Cart new.! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/10/2012 17:40'!
test01aCartShouldBeEmptyWhenCreated

	self assert: (self newCart size = 0).! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/10/2012 18:53'!
test02aCartShouldContainABookAdded

	|aCart aBook|

	aBook := self newBook.
	aCart:= self newCart. 
	aCart add: aBook.
	self assert:  (aCart includes: aBook).
	
! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/10/2012 20:17'!
test03aCartShouldContainTheSameBooksPlusTheBookAdded

	|aCart1  aCart2 aBook1 aBook2|
	
	aBook2 := self newBook.
	aBook1 := self newBook.
	aCart1:= self newCart.
	aCart1 add: aBook1.
	aCart2 := aCart1 copy.
	aCart1 add: aBook2.
	
	self assert: (aCart1 = (aCart2 add: aBook2)).! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/10/2012 19:04'!
test04aCartShouldContainTheSameQuantityOfBookAdded

	|aCart aBook|
	
	aBook := self newBook.
	aCart:= self newCart.
	aCart add: aBook withOccurrences: 5 .
	self assert: ((aCart occurrencesOf: aBook) = 5).! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/10/2012 16:47'!
testCatalogShouldBeEmptyWhenJustCreated

	|catalog|
	
	catalog := Catalog new.
		
	self assert: (catalog isEmpty).! !
